@media (max-width: 400px){
    th#rstatus,
    td[name='rstatus'],
    th#type,
    td[name='type']{
        display: none;
    }
}
@media (max-width: 600px){
    th#subject,
    td[name='subject']{
        display: none;
    }
}
@media (max-width: 767px){
    th#to,
    td[name='to']{
        display: none;
    }
}

@media (max-width: 990px){
    #filters_bar,
    th#date,
    td[name='date']{
        display: none;
    }
}

#menubar {
    position: absolute;
    top: 51px;
    padding-left: 15px;
    padding-right: 15px;
    padding-top: 20px;
    padding-bottom: 10px;
    left: 0;
    right: 0;
    margin: 0;
    z-index: 10;
}

#menubar input[type="checkbox"] {
    margin: 0;
}

#listing {
    position: absolute;
    top: 125px;
    left: 0;
    right: 0;
    bottom: 0;
    margin: 0;
    overflow: auto;
    padding-left: 15px;
    padding-right: 15px;
}

#mailcontent {
    border: 0;
    margin: 0;
    padding: 0;
    width: 100%;
    height: 100%;
}

#emails tbody tr:hover {
    color: #0088cc;
}

thead tr {
    position: relative;
    z-index: 1;
}

td[name=type] span {
    cursor: pointer;
}

td.unseen {
    font-weight: bold;
}

td[class*=pending] {
    font-style: italic;
}

td[class*=openable] {
    cursor: pointer;
}