#menubar { 
    padding-left: 15px;
    z-index: 10;
}

#listing { 
    position: absolute;
    padding: 0px;
    right: 0;
    left: 0;
    bottom: 0;
    overflow: hidden;
}

#mailcontent {
    border: 0;
    margin: 0;
    padding: 0;
    width: 100%;
    height: 100%;
}