{% extends "fluid.html" %}{% load i18n amavis_tags %}
{% load static %}

{% block pagetitle %}{% trans "Quarantine management" %}{% endblock %}

{% block extra_css %}
<link rel="stylesheet" type="text/css" href="{% static 'modoboa_amavis/css/quarantine.css' %}" />
<link rel="stylesheet" type="text/css" href="{% static 'css/jquery.sortable.css' %}" />
{% endblock %}

{% block extra_js %}
<script type="text/javascript" src="{% static 'js/jquery.htmltable.js' %}"></script>
<script type="text/javascript" src="{% static 'js/jquery.infinitescroll.js' %}"></script>
<script type="text/javascript" src="{% static 'js/jquery.sortable.js' %}"></script>
<script type="text/javascript" src="{% static 'js/history.js' %}"></script>
<script type="text/javascript" src="{% static 'js/searchbar.js' %}"></script>
<script type="text/javascript" src="{% static 'js/listing.js' %}"></script>
<script type="text/javascript" src="{% static 'modoboa_amavis/js/quarantine.js' %}"></script>
<script type="text/javascript">
$(document).ready(function() {
    var qm = new Quarantine({
        deflocation: "listing/?sort_order=-date&reset_page=true",
        defcallback: "listing_cb",
        load_page_url: "{% url 'modoboa_amavis:mail_page' %}",
        check_learning_rcpt: {{ check_learning_rcpt }},
        learning_recipient_url: "{% url 'modoboa_amavis:learning_recipient_set' %}"
    });
});
</script>
{% endblock %}

{% block container_content %}
<div id="menubar">
  {% quar_menu user %}
</div>
<div id="listing"></div>
{% endblock %}
