{% load i18n form_tags %}
{% render_field form.bypass_virus_checks %}
{% render_field form.bypass_spam_checks %}
<div class="form-group">
  <label class="control-label col-lg-4 col-md-4 col-sm-4">{% trans "Thresholds" %}</label>
  <div class="col-lg-2 col-md-2 col-sm-2">
    {{ form.spam_tag2_level }}
  </div>
  <span class="help-block">{% trans "or more is spam" %}</span>
</div>
<div class="form-group">
  <div class="col-sm-offset-4 col-lg-2 col-md-2 col-sm-2">
    {{ form.spam_kill_level }}
  </div>
  <span class="help-block">{% trans "or more throw spam message away" %}</span>
</div>
{% render_field form.spam_subject_tag2 activator=True activator_label=_("default") activator_size="col-sm-3" %}
{% render_field form.bypass_banned_checks %}
